import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import AuthRefreshResponse

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/auth-refresh-response.json")
OBJ_DATA = AuthRefreshResponse.from_json(JSON_DATA)


def assert_auth_refresh_response(resp: AuthRefreshResponse):
    assert resp.access_token == DICT_DATA["jwtTokens"]["access"]
    assert resp.refresh_token == DICT_DATA["jwtTokens"]["refresh"]


def test_auth_refresh_response_from_valid_json():
    resp = AuthRefreshResponse.from_json(JSON_DATA)
    assert_auth_refresh_response(resp)


def test_auth_refresh_response_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthRefreshResponse.from_json("{}")


def test_auth_refresh_response_from_valid_dict():
    resp = AuthRefreshResponse.from_dict(DICT_DATA)
    assert_auth_refresh_response(resp)


def test_auth_refresh_response_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthRefreshResponse.from_dict({})


def test_auth_refresh_response_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_auth_refresh_response_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA



"""This module contains the GroupOperation domain model."""

from typing import Dict

from mythx_models.exceptions import ValidationError
from mythx_models.request.base import BaseRequest

GROUP_OPERATION_KEYS = ("type", "group_id")


class GroupOperationRequest(BaseRequest):
    """Perform an API request that performs an action on the specified group
    ID."""

    def __init__(self, group_id: str, type_: str):
        self.group_id = group_id
        self.type = type_

    @property
    def method(self) -> str:
        """The HTTP method to perform.

        :return: The uppercase HTTP method, e.g. "POST"
        """
        return "POST"

    @property
    def endpoint(self) -> str:
        """The API's group status endpoint.

        :return: A string denoting the status endpoint without the host prefix
        """
        return "v1/analysis-groups/{}".format(self.group_id)

    @property
    def headers(self) -> Dict:
        """Additional request headers.

        :return: A dict (str -> str) instance mapping header name to header content
        """
        return {}

    @property
    def parameters(self) -> Dict:
        """Additional URL parameters.

        :return: A dict (str -> str) instance mapping parameter name to parameter content
        """
        return {}

    @property
    def payload(self) -> Dict:
        """The request's payload data.

        :return: A Python dict to be serialized into JSON format and submitted to the endpoint.
        """
        return {"type": self.type}

    @classmethod
    def from_dict(cls, d) -> "GroupOperationRequest":
        """Create the request domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        if not all(k in d for k in GROUP_OPERATION_KEYS):
            raise ValidationError(
                "Not all required keys {} found in data {}".format(
                    GROUP_OPERATION_KEYS, d
                )
            )
        # TODO: Validate UUID and correct type
        return cls(group_id=d["group_id"], type_=d["type"])

    def to_dict(self) -> Dict:
        """Serialize the request model to a Python dict.

        :return: A dict holding the request model data
        """
        return {"group_id": self.group_id, "type": self.type}

    def __eq__(self, other: "GroupOperationRequest") -> bool:
        return all((self.group_id == other.group_id, self.type == other.type))



"""This module contains the AnalysisInputResponse domain model."""

from mythx_models.request.analysis_submission import AnalysisSubmissionRequest


class AnalysisInputResponse(AnalysisSubmissionRequest):
    pass



"""This module contains the GroupStatusResponse domain model."""

import json
from typing import Any, Dict

from mythx_models.response.base import BaseResponse
from mythx_models.response.group import Group
from mythx_models.util import resolve_schema


class GroupStatusResponse(BaseResponse):
    """The API response domain model for a fetching the status of a group."""

    with open(resolve_schema(__file__, "group-status.json")) as sf:
        schema = json.load(sf)

    def __init__(self, group: Group):
        self.group = group

    @classmethod
    def from_dict(cls, d) -> "GroupStatusResponse":
        """Create the response domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        cls.validate(d)
        return cls(group=Group.from_dict(d))

    def to_dict(self) -> Dict:
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """
        d = self.group.to_dict()
        self.validate(d)
        return d

    def __getattr__(self, name: str) -> Any:
        return getattr(self.group, name)

    def __eq__(self, other: "GroupStatusResponse") -> bool:
        return self.group == other.group



"""This module contains the GroupOperationResponse domain model."""

from mythx_models.response.group_status import GroupStatusResponse


class GroupOperationResponse(GroupStatusResponse):
    """The API response domain model for the status of a group after the
    operation."""

    pass



"""This module contains domain models regarding analysis jobs."""

import logging
from enum import Enum
from typing import Dict

from inflection import underscore

from mythx_models.response.base import BaseResponse
from mythx_models.response.group import VulnerabilityStatistics
from mythx_models.util import (
    deserialize_api_timestamp,
    dict_delete_none_fields,
    serialize_api_timestamp,
)

LOGGER = logging.getLogger(__name__)


class AnalysisStatus(str, Enum):
    """An Enum describing the status an analysis job can be in."""

    QUEUED = "Queued"
    IN_PROGRESS = "In Progress"
    ERROR = "Error"
    FINISHED = "Finished"


class Analysis(BaseResponse):
    """An object describing an analysis job.

    Such a model was built, because many other API responses deliver the
    same data when it comes to analysis jobs. This makes the code more
    DRY, validation easier, and allows for recursive SerDe (e.g. mapping
    :code:`from_dict` to a deserialized JSON list of job objects.
    """

    def __init__(
        self,
        uuid: str,
        api_version: str,
        mythril_version: str,
        harvey_version: str,
        maru_version: str,
        queue_time: int,
        status: AnalysisStatus,
        submitted_at: str,
        submitted_by: str,
        main_source: str = None,
        num_sources: int = None,
        vulnerability_statistics: Dict[str, int] = None,
        run_time: int = 0,
        client_tool_name: str = None,
        error: str = None,
        info: str = None,
        group_id: str = None,
        analysis_mode: str = None,
        group_name: str = None,
        upgraded: bool = None,
        property_checking: bool = None,
        *args,
        **kwargs
    ):
        if vulnerability_statistics is None:
            self.vulnerability_statistics = None
        else:
            self.vulnerability_statistics = VulnerabilityStatistics.from_dict(
                vulnerability_statistics
            )

        self.uuid = uuid
        self.api_version = api_version
        self.mythril_version = mythril_version
        self.harvey_version = harvey_version
        self.maru_version = maru_version
        self.queue_time = queue_time
        self.run_time = run_time
        self.status = AnalysisStatus(status.title())
        self.submitted_at = deserialize_api_timestamp(submitted_at)
        self.submitted_by = submitted_by
        self.main_source = main_source
        self.num_sources = num_sources
        self.client_tool_name = client_tool_name
        self.error = error
        self.info = info
        self.group_id = group_id
        self.group_name = group_name
        self.analysis_mode = analysis_mode
        self.upgraded = upgraded
        self.property_checking = property_checking

        if args or kwargs:
            LOGGER.debug(
                "Got unexpected arguments args={}, kwargs={}".format(args, kwargs)
            )

    @classmethod
    def from_dict(cls, d) -> "Analysis":
        """Create the response domain model from a dict.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        d = {underscore(k): v for k, v in d.items()}
        if "num_vulnerabilities" in d:
            d["vulnerability_statistics"] = d.pop("num_vulnerabilities")
        return cls(**d)

    def to_dict(self) -> Dict:
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {
            "uuid": self.uuid,
            "apiVersion": self.api_version,
            "mythrilVersion": self.mythril_version,
            "harveyVersion": self.harvey_version,
            "maruVersion": self.maru_version,
            "queueTime": self.queue_time,
            "runTime": self.run_time,
            "status": self.status.title(),
            "submittedAt": serialize_api_timestamp(self.submitted_at),
            "submittedBy": self.submitted_by,
            "mainSource": self.main_source,
            "numSources": self.num_sources,
            "numVulnerabilities": self.vulnerability_statistics.to_dict()
            if self.vulnerability_statistics
            else None,
            "clientToolName": self.client_tool_name,
            "analysisMode": self.analysis_mode,
            "groupName": self.group_name,
            "groupId": self.group_id,
        }
        if self.error is not None:
            d.update({"error": self.error})
        if self.info is not None:
            d.update({"info": self.error})

        return dict_delete_none_fields(d)

    def __eq__(self, candidate: "Analysis") -> bool:
        return all(
            (
                self.uuid == candidate.uuid,
                self.api_version == candidate.api_version,
                self.mythril_version == candidate.mythril_version,
                self.harvey_version == candidate.harvey_version,
                self.maru_version == candidate.maru_version,
                self.queue_time == candidate.queue_time,
                self.run_time == candidate.run_time,
                self.status == candidate.status,
                self.submitted_at == candidate.submitted_at,
                self.submitted_by == candidate.submitted_by,
                self.main_source == candidate.main_source,
                self.num_sources == candidate.num_sources,
                self.vulnerability_statistics == candidate.vulnerability_statistics,
                self.client_tool_name == candidate.client_tool_name,
                self.error == candidate.error,
                self.info == candidate.info,
                self.group_id == candidate.group_id,
                self.analysis_mode == candidate.analysis_mode,
                self.group_name == candidate.group_name,
            )
        )

    def __repr__(self) -> str:
        return "<Analysis uuid={} status={}>".format(self.uuid, self.status)



import json
from copy import deepcopy

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import AnalysisListResponse
from mythx_models.response.analysis import Analysis
from mythx_models.util import serialize_api_timestamp

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/analysis-list-response.json")
OBJ_DATA = AnalysisListResponse.from_json(JSON_DATA)


def assert_analysis_data(expected, analysis: Analysis):
    assert expected["apiVersion"] == analysis.api_version
    assert expected["maruVersion"] == analysis.maru_version
    assert expected["mythrilVersion"] == analysis.mythril_version
    assert expected["harveyVersion"] == analysis.harvey_version
    assert expected["queueTime"] == analysis.queue_time
    assert expected["runTime"] == analysis.run_time
    assert expected["status"] == analysis.status
    assert expected["submittedAt"] == serialize_api_timestamp(analysis.submitted_at)
    assert expected["submittedBy"] == analysis.submitted_by
    assert expected["uuid"] == analysis.uuid
    assert expected["groupName"] == analysis.group_name
    assert expected["groupId"] == analysis.group_id
    assert expected["analysisMode"] == analysis.analysis_mode


def test_analysis_list_from_valid_json():
    assert len(OBJ_DATA.analyses) == 2
    for i, analysis in enumerate(OBJ_DATA.analyses):
        assert_analysis_data(DICT_DATA["analyses"][i], analysis)


def test_analysis_list_from_invalid_json():
    with pytest.raises(ValidationError):
        AnalysisListResponse.from_json("[]")


def test_analysis_list_from_empty_json():
    with pytest.raises(ValidationError):
        AnalysisListResponse.from_json("{}")


def test_analysis_list_from_valid_dict():
    assert len(OBJ_DATA.analyses) == 2
    for i, analysis in enumerate(OBJ_DATA.analyses):
        assert_analysis_data(DICT_DATA["analyses"][i], analysis)


def test_analysis_list_from_invalid_dict():
    with pytest.raises(ValidationError):
        AnalysisListResponse.from_dict("[]")


def test_analysis_list_from_empty_dict():
    with pytest.raises(ValidationError):
        AnalysisListResponse.from_dict({})


def test_analysis_list_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA


def test_analysis_list_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_iteration():
    uuids = (
        "0680a1e2-b908-4c9a-a15b-636ef9b61486",
        "0680a1e2-b908-4c9a-a15b-636ef9b61487",
    )
    # XXX: Don't use zip here to make sure __iter__ returns
    for idx, analysis in enumerate(OBJ_DATA):
        assert uuids[idx] == analysis.uuid


def test_valid_getitem():
    for idx, analysis in list(enumerate(OBJ_DATA.analyses)):
        assert OBJ_DATA[idx] == analysis


def test_invalid_getitem_index():
    with pytest.raises(IndexError):
        OBJ_DATA[1337]


def test_oob_getitem_slice():
    assert OBJ_DATA[1337:9001] == []


def test_valid_delitem():
    analysis_list = deepcopy(OBJ_DATA)
    del analysis_list[0]
    assert analysis_list.analyses == OBJ_DATA[1:]
    assert analysis_list.total == OBJ_DATA.total - 1
    assert len(analysis_list) == OBJ_DATA.total - 1


def test_invalid_delitem():
    with pytest.raises(IndexError):
        del OBJ_DATA[1337]


def test_valid_setitem():
    analysis_list = deepcopy(OBJ_DATA)
    analysis_list[0] = "foo"
    assert analysis_list.analyses[0] == "foo"
    assert analysis_list[0] == "foo"
    assert len(analysis_list.analyses) == len(OBJ_DATA.analyses)


def test_invalid_setitem():
    with pytest.raises(IndexError):
        OBJ_DATA[1337] = "foo"


def test_reversed():
    analysis_list = deepcopy(OBJ_DATA)
    assert list(reversed(analysis_list)) == list(reversed(OBJ_DATA.analyses))
    assert analysis_list.total == OBJ_DATA.total


def test_valid_object_contains():
    assert OBJ_DATA.analyses[0] in OBJ_DATA


def test_valid_uuid_contains():
    assert OBJ_DATA.analyses[0].uuid in OBJ_DATA


def test_contains_error():
    with pytest.raises(ValueError):
        1 in OBJ_DATA


def test_invalid_object_contains():
    analysis = deepcopy(OBJ_DATA.analyses[0])
    analysis.uuid = "something else"
    assert analysis not in OBJ_DATA


def test_invalid_uuid_contains():
    assert "foo" not in OBJ_DATA


def test_list_not_equals():
    analysis_list = deepcopy(OBJ_DATA)
    analysis_list.total = 0
    assert analysis_list != OBJ_DATA


def test_nested_analysis_not_equals():
    analysis_list = deepcopy(OBJ_DATA)
    analysis_list.analyses[0].uuid = "foo"
    assert analysis_list != OBJ_DATA


def test_valid_equals():
    analysis_list = deepcopy(OBJ_DATA)
    assert analysis_list == OBJ_DATA



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import AuthLoginResponse

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/auth-login-response.json")
OBJ_DATA = AuthLoginResponse.from_json(JSON_DATA)


def assert_auth_login_response(resp: AuthLoginResponse):
    assert resp.api_key == DICT_DATA["jwtTokens"]["access"]
    assert resp.refresh_token == DICT_DATA["jwtTokens"]["refresh"]


def test_auth_login_response_from_valid_json():
    resp = AuthLoginResponse.from_json(JSON_DATA)
    assert_auth_login_response(resp)


def test_auth_login_response_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthLoginResponse.from_json("{}")


def test_auth_login_response_from_valid_dict():
    resp = AuthLoginResponse.from_dict(DICT_DATA)
    assert_auth_login_response(resp)


def test_auth_login_response_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthLoginResponse.from_dict({})


def test_auth_login_response_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_auth_login_response_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA



"""This module contains the AnalysisSubmissionRequest domain model."""

import json
import logging
from typing import Dict, List

from mythx_models.request.base import BaseRequest
from mythx_models.util import dict_delete_none_fields, resolve_schema

LOGGER = logging.getLogger(__name__)


class AnalysisSubmissionRequest(BaseRequest):
    """Perform an API analysis job submission as a logged in user."""

    with open(resolve_schema(__file__, "analysis-submission.json")) as sf:
        schema = json.load(sf)

    def __init__(
        self,
        contract_name: str = None,
        bytecode: str = None,
        source_map: str = None,
        deployed_bytecode: str = None,
        deployed_source_map: str = None,
        main_source: str = None,
        sources: Dict[str, Dict[str, str]] = None,
        source_list: List[str] = None,
        solc_version: str = None,
        analysis_mode: str = "quick",
    ):
        self.contract_name = contract_name
        self.bytecode = bytecode
        self.source_map = source_map
        self.deployed_bytecode = deployed_bytecode
        self.deployed_source_map = deployed_source_map
        self.main_source = main_source
        self.sources = sources
        self.source_list = source_list
        self.solc_version = solc_version
        # set alias for full mode for backwards compatibility - new modes are quick, standard, deep
        self.analysis_mode = self._get_analysis_mode(analysis_mode)

    @staticmethod
    def _get_analysis_mode(mode: str) -> str:
        return "standard" if mode == "full" else mode

    @property
    def endpoint(self) -> str:
        """The API's analysis submission endpoint.

        :return: A string denoting the submission endpoint without the host prefix
        """
        return "v1/analyses"

    @property
    def method(self) -> str:
        """The HTTP method to perform.

        :return: The uppercase HTTP method, e.g. "POST"
        """
        return "POST"

    @property
    def parameters(self) -> Dict:
        """Additional URL parameters.

        :return: A dict (str -> str) instance mapping parameter name to parameter content
        """
        return {}

    @property
    def headers(self) -> Dict:
        """Additional request headers.

        :return: A dict (str -> str) instance mapping header name to header content
        """
        return {}

    @property
    def payload(self) -> Dict:
        """The request's payload data.

        :return: A Python dict to be serialized into JSON format and submitted to the endpoint.
        """
        return {"data": self.to_dict()}

    @classmethod
    def from_dict(cls, d: Dict) -> "AnalysisSubmissionRequest":
        """Create the request domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        if d.get("analysisMode"):
            d["analysisMode"] = cls._get_analysis_mode(d["analysisMode"])

        cls.validate(d)
        return cls(
            contract_name=d.get("contractName"),
            bytecode=d.get("bytecode"),
            source_map=d.get("sourceMap"),
            deployed_bytecode=d.get("deployedBytecode"),
            deployed_source_map=d.get("deployedSourceMap"),
            main_source=d.get("mainSource"),
            sources=d.get("sources"),
            source_list=d.get("sourceList"),
            solc_version=d.get("version"),
            analysis_mode=d.get("analysisMode"),
        )

    def to_dict(self) -> Dict:
        """Serialize the request model to a Python dict.

        :return: A dict holding the request model data
        """
        base_dict = dict_delete_none_fields(
            {
                "contractName": self.contract_name,
                "bytecode": self.bytecode,
                "sourceMap": self.source_map,
                "deployedBytecode": self.deployed_bytecode,
                "deployedSourceMap": self.deployed_source_map,
                "mainSource": self.main_source,
                "sources": self.sources if self.sources else None,
                "sourceList": self.source_list if self.source_list else None,
                "version": self.solc_version,
                "analysisMode": self.analysis_mode,
            }
        )
        self.validate(base_dict)
        return base_dict

    def __eq__(self, candidate) -> bool:
        """Perform an equality check on two Request domain models."""

        return all(
            [
                self.contract_name == candidate.contract_name,
                self.bytecode == candidate.bytecode,
                self.source_map == candidate.source_map,
                self.deployed_bytecode == candidate.deployed_bytecode,
                self.deployed_source_map == candidate.deployed_source_map,
                self.main_source == candidate.main_source,
                self.sources == candidate.sources,
                self.source_list == candidate.source_list,
                self.solc_version == candidate.solc_version,
                self.analysis_mode == candidate.analysis_mode,
            ]
        )



"""This module contains exceptions raised by MythX."""


class MythXBaseException(Exception):
    """A base exception describing MythX-related errors."""

    pass


class ValidationError(ValueError):
    """An exception denoting an error in SerDe of request or response data."""

    pass


class MythXAPIError(MythXBaseException):
    """An exception denoting an API-related error.

    This is usually raised when the API takes too long to respond, or a
    response contains an HTTP status code that is not 200 OK. In this
    case, the exception is passed on to the developer. This should give
    them early warnings about malformed data on their side, or recover
    in case the API is not available or experiences some kind of error
    we cannot handle.
    """

    pass



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import Analysis, AnalysisSubmissionResponse
from mythx_models.util import serialize_api_timestamp

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/analysis-submission-response.json")
OBJ_DATA = AnalysisSubmissionResponse.from_json(JSON_DATA)


def assert_analysis_data(expected, analysis: Analysis):
    assert expected["apiVersion"] == analysis.api_version
    assert expected["maruVersion"] == analysis.maru_version
    assert expected["mythrilVersion"] == analysis.mythril_version
    assert expected["harveyVersion"] == analysis.harvey_version
    assert expected["queueTime"] == analysis.queue_time
    assert expected["runTime"] == analysis.run_time
    assert expected["status"] == analysis.status
    assert expected["submittedAt"] == serialize_api_timestamp(analysis.submitted_at)
    assert expected["submittedBy"] == analysis.submitted_by
    assert expected["uuid"] == analysis.uuid


def test_analysis_submission_from_valid_json():
    resp = AnalysisSubmissionResponse.from_json(JSON_DATA)
    assert_analysis_data(DICT_DATA, resp.analysis)


def test_analysis_submission_from_empty_json():
    with pytest.raises(ValidationError):
        AnalysisSubmissionResponse.from_json("{}")


def test_analysis_submission_from_valid_dict():
    resp = AnalysisSubmissionResponse.from_dict(DICT_DATA)
    assert_analysis_data(DICT_DATA, resp.analysis)


def test_analysis_submission_from_empty_dict():
    with pytest.raises(ValidationError):
        AnalysisSubmissionResponse.from_dict({})


def test_analysis_submission_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA


def test_analysis_submission_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_analysis_submission_property_delegation():
    assert_analysis_data(DICT_DATA, OBJ_DATA)



from mythx_models.request import VersionRequest

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/version-request.json")
OBJ_DATA = VersionRequest.from_json(JSON_DATA)


def assert_version_request(req):
    assert req.method == "GET"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == {}
    assert req.endpoint == "v1/version"


def test_version_request_from_valid_json():
    req = VersionRequest.from_json("{}")
    assert_version_request(req)


def test_version_request_from_valid_dict():
    req = VersionRequest.from_dict({})
    assert_version_request(req)


def test_version_request_to_json():
    assert OBJ_DATA.to_json() == "{}"


def test_version_request_to_dict():
    assert OBJ_DATA.to_dict() == {}



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import AuthLoginRequest

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/auth-login-request.json")
OBJ_DATA = AuthLoginRequest.from_json(JSON_DATA)


def assert_login(req: AuthLoginRequest):
    assert req.username == DICT_DATA["username"]
    assert req.password == DICT_DATA["password"]
    assert req.method == "POST"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == DICT_DATA


def test_login_from_valid_json():
    req = AuthLoginRequest.from_json(JSON_DATA)
    assert_login(req)


def test_login_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthLoginRequest.from_json("{}")


def test_login_from_valid_dict():
    req = AuthLoginRequest.from_dict(DICT_DATA)
    assert_login(req)


def test_login_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthLoginRequest.from_dict({})


def test_login_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_login_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA
